@@ -16,15 +16,40 @@
 
 #include "xtensor.hpp"
 #include "xsort.hpp"
+#include "xset_operation.hpp"
+#include "xview.hpp"
+
+using namespace xt::placeholders;
 
 namespace xt
 {
+    /**
+     * @ingroup digitize
+     * @brief Return the indices of the bins to which each value in input array belongs.
+     *
+     * @param data The data.
+     * @param bin_edges The bin-edges. It has to be 1-dimensional and monotonic.
+     * @param right Indicating whether the intervals include the right or the left bin edge.
+     * @return Output array of indices, of same shape as x.
+     */
+    template <class E1, class E2>
+    inline auto digitize(E1&& data, E2&& bin_edges, bool right = false)
+    {
+        XTENSOR_ASSERT(bin_edges.dimension() == 1);
+        XTENSOR_ASSERT(bin_edges.size() >= 2);
+        XTENSOR_ASSERT(std::is_sorted(bin_edges.cbegin(), bin_edges.cend()));
+        XTENSOR_ASSERT(xt::amin(data)[0] >= bin_edges[0]);
+        XTENSOR_ASSERT(xt::amax(data)[0] <= bin_edges[bin_edges.size() - 1]);
+
+        return xt::searchsorted(std::forward<E2>(bin_edges), std::forward<E1>(data), right);
+    }
+
     /**
      * @ingroup histogram
      * @brief Compute the histogram of a set of data.
      *
      * @param data The data.
-     * @param bin_edges The bin-edges.
+     * @param bin_edges The bin-edges. It has to be 1-dimensional and monotonic.
      * @param weights Weight factors corresponding to each data-point.
      * @param density If true the resulting integral is normalized to 1. [default: false]
      * @return An one-dimensional xarray<double>, length: bin_edges.size()-1.
@@ -32,54 +57,38 @@ namespace xt
     template <class R = double, class E1, class E2, class E3>
     inline auto histogram(E1&& data, E2&& bin_edges, E3&& weights, bool density = false)
     {
-        // alias counter and value type
         using size_type = common_size_type_t<std::decay_t<E1>, std::decay_t<E2>, std::decay_t<E3>>;
         using value_type = typename std::decay_t<E3>::value_type;
 
-        // basic checks
-        // - rank
         XTENSOR_ASSERT(data.dimension() == 1);
         XTENSOR_ASSERT(weights.dimension() == 1);
         XTENSOR_ASSERT(bin_edges.dimension() == 1);
-        // - size
         XTENSOR_ASSERT(weights.size() == data.size());
         XTENSOR_ASSERT(bin_edges.size() >= 2);
-        // - bin-edges must be sorted
         XTENSOR_ASSERT(std::is_sorted(bin_edges.cbegin(), bin_edges.cend()));
-        // - data must be enclosed in the bins
         XTENSOR_ASSERT(xt::amin(data)[0] >= bin_edges[0]);
         XTENSOR_ASSERT(xt::amax(data)[0] <= bin_edges[bin_edges.size() - 1]);
 
-        // initialize output
         xt::xtensor<value_type, 1> count = xt::zeros<value_type>({ bin_edges.size() - 1 });
 
-        // indices that sort "data"
-        auto isort = xt::argsort(data);
+        auto sorter = xt::argsort(data);
 
-        // index of the current bin
         size_type ibin = 0;
 
-        // fill the histogram: loop over (sorted) data
-        for (auto& idx : isort)
+        for (auto& idx : sorter)
         {
-            // - proceed to the relevant bin
             while (data[idx] >= bin_edges[ibin + 1] && ibin < bin_edges.size() - 2)
             {
                 ++ibin;
             }
-            // - update the count
             count[ibin] += weights[idx];
         }
 
-        // cast type
         xt::xtensor<R, 1> prob = xt::cast<R>(count);
 
-        // normalize
         if (density)
         {
-            // - size as doubles
             R n = static_cast<R>(data.size());
-            // - apply normalization
             for (size_type i = 0; i < bin_edges.size() - 1; ++i)
             {
                 prob[i] /= (static_cast<R>(bin_edges[i + 1] - bin_edges[i]) * n);
@@ -244,7 +253,7 @@ namespace xt
             case histogram_algorithm::uniform:
             {
                 // indices that sort "data"
-                auto isort = xt::argsort(data);
+                auto sorter = xt::argsort(data);
 
                 // histogram: all of equal 'height'
                 // - height
@@ -272,13 +281,15 @@ namespace xt
                 {
                     if (cum_weight >= count[ibin])
                     {
-                        bin_edges[ibin + 1] = data[isort[i]];
+                        bin_edges[ibin + 1] = data[sorter[i]];
                         ++ibin;
                     }
-                    cum_weight += weights[isort[i]];
+                    cum_weight += weights[sorter[i]];
                 }
                 return bin_edges;
             }
+
+            // bins of equal width
             default:
             {
                 xt::xtensor<value_type, 1> bin_edges
@@ -426,6 +437,61 @@ namespace xt
                         xt::ones<typename std::decay_t<E1>::value_type>(data.shape()),
                         minlength);
     }
+
+    /**
+     * Get the number of items in each bin, given the fraction of items per bin.
+     * The output is such that the total number of items of all bins is exactly "N".
+     *
+     * @param N the number of items to distribute
+     * @param weights fraction of items per bin: a 1D container whose size is the number of bins
+     *
+     * @return 1D container with the number of items per bin
+     */
+    template <class E>
+    inline xt::xtensor<size_t, 1> bin_items(size_t N, E&& weights)
+    {
+        if (weights.size() <= std::size_t(1))
+        {
+            xt::xtensor<size_t, 1> n = N * xt::ones<size_t>({1});
+            return n;
+        }
+
+        using value_type = typename std::decay_t<E>::value_type;
+
+        XTENSOR_ASSERT(xt::all(weights >= static_cast<value_type>(0)));
+        XTENSOR_ASSERT(xt::sum(weights)() > static_cast<value_type>(0));
+
+        xt::xtensor<double, 1> P = xt::cast<double>(weights) / static_cast<double>(xt::sum(weights)());
+        xt::xtensor<size_t, 1> n = xt::ceil(static_cast<double>(N) * P);
+
+        if (xt::sum(n)() == N)
+        {
+            return n;
+        }
+
+        xt::xtensor<size_t, 1> d = xt::zeros<size_t>(P.shape());
+        xt::xtensor<size_t, 1> sorter = xt::argsort(P);
+        sorter = xt::view(sorter, xt::range(P.size(), _, -1));
+        sorter = xt::view(sorter, xt::range(0, xt::sum(n)(0) - N));
+        xt::view(d, xt::keep(sorter)) = 1;
+        n -= d;
+
+        return n;
+    }
+
+    /**
+     * Get the number of items in each bin, with each bin having approximately the same number of
+     * items in it,under the constraint that the total number of items of all bins is exactly "N".
+     *
+     * @param N the number of items to distribute
+     * @param bins the number of bins
+     *
+     * @return 1D container with the number of items per bin
+     */
+    inline xt::xtensor<size_t,1> bin_items(size_t N, size_t bins)
+    {
+        return bin_items(N, xt::ones<double>({bins}));
+    }
 }
 
 #endif
