@@ -201,6 +201,8 @@ namespace xt
      * accumulate implementation *
      *****************************/
 
+    /// @cond DOXYGEN_INCLUDE_NOEXCEPT
+
     namespace detail
     {
         template <std::size_t I, class F, class R, class... T>
@@ -227,6 +229,8 @@ namespace xt
         return detail::accumulate_impl<0, F, R, T...>(std::forward<F>(f), init, t);
     }
 
+    /// @endcond
+
     /***************************
      * argument implementation *
      ***************************/
@@ -609,6 +613,27 @@ namespace xt
     {
     };
 
+    /******************************
+     * is_iterator implementation *
+     ******************************/
+
+    template <class E, class = void>
+    struct is_iterator : std::false_type
+    {
+    };
+
+    template <class E>
+    struct is_iterator<E, void_t<decltype(
+        *std::declval<const E>(),
+        std::declval<const E>() == std::declval<const E>(),
+        std::declval<const E>() != std::declval<const E>(),
+        ++ (*std::declval<E*>()),
+        (*std::declval<E*>()) ++,
+        std::true_type())>>
+        : std::true_type
+    {
+    };
+
     /********************************************
      * xtrivial_default_construct implemenation *
      ********************************************/
@@ -832,6 +857,19 @@ namespace xt
 
     template <class ST>
     using inner_reference_t = typename inner_reference<ST>::type;
+
+    /************
+     * has_rank *
+     ************/
+
+    template <class E, size_t N>
+    struct has_rank
+    {
+        using type = std::integral_constant<bool, std::decay_t<E>::rank == N>;
+    };
+
+    template <class E, size_t N>
+    using has_rank_t = typename has_rank<std::decay_t<E>, N>::type;
 }
 
 #endif
